using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;

namespace Feel
{
    public class LiquidGlassBottle : MVRScript
    {
        #region PluginInfo    
        public string pluginAuthor = "Feel"; // edited by VamTastic (original by Frief )
        public string pluginName = "LiquidGlassPack";
        public string pluginVersion = "1";
        public string pluginDate = "23/09" + "/2022";
        public string pluginDescription = @"Liquid GlassPack Bottle";
        #endregion

        #region Vars    

        protected bool first_loading = false;

        protected JSONStorableColor glassColor;
        protected JSONStorableColor CapColor;
        protected JSONStorableColor BlandColor;

        protected JSONStorableFloat GlassTrans;
        protected JSONStorableBool GlassTransComponents;
        protected JSONStorableBool LiquidPhysicsComponents;
        protected JSONStorableBool CapOnOffComponents;
        protected JSONStorableBool BrandOnOffComponents;

        private string CurrType = null;

        public Transform ObjectRoot = null;
        public Transform LiquidRoot;
        public Transform GlassRoot;
        public Transform SubGlassRoot;
        public Transform GlassCapRoot;
        public Transform GlassSubCapRoot;
        public Transform GlassbrandRoot;

        private bool editorInitialized = false;
        private int initLoops = 0;
        private int maxInitLoops = 1500; // 3600 for one minute

        private Renderer rend = null;

        private float CurrScale = 1f;
        private float CurrAngle = 0f;

        private LiquidOpt _liquid_opt;

        #endregion

        public override void Init()
        {
            try
            {
                if (containingAtom.type != "CustomUnityAsset")
                {
                    SuperController.LogError("Liquid Glass Pack Bottle - Please add this script on a Custom Unity Asset ( script disabled).");
                    return;
                }

                _liquid_opt = new LiquidOpt();
                _liquid_opt.Init(this);

                GlassTransComponents = new JSONStorableBool("Glass Transparent", true, ChangeGlassRenderModeCallback);
                RegisterBool(GlassTransComponents);
                UIDynamicToggle toggle1 = CreateToggle(GlassTransComponents, true);
                toggle1.label = "Glass Transparent On Off";
                //toggle1.height = 80;

                LiquidPhysicsComponents = new JSONStorableBool("Liquid Physics", true, ChangeLiquidPhysicsCallback);
                RegisterBool(LiquidPhysicsComponents);
                UIDynamicToggle toggle2 = CreateToggle(LiquidPhysicsComponents, true);
                toggle2.label = "Liquid Physics On Off";
                //toggle2.height = 80;

                CapOnOffComponents = new JSONStorableBool("Cap OnOff", true, ChangeCapOnOffCallback);
                RegisterBool(CapOnOffComponents);
                UIDynamicToggle toggle3 = CreateToggle(CapOnOffComponents, true);
                toggle3.label = "Cap On Off";
                //toggle3.height = 80;

                BrandOnOffComponents = new JSONStorableBool("Brand OnOff", true, ChangeBrandOnOffCallback);
                RegisterBool(BrandOnOffComponents);
                UIDynamicToggle toggle4 = CreateToggle(BrandOnOffComponents, true);
                toggle4.label = "Brand On Off";
                //toggle3.height = 80;

                //**************  glass Alpha
                GlassTrans = new JSONStorableFloat("Glass Alpha", 0.3f, OnGlassAlphaChanged, 0f, 0.7f);
                RegisterFloat(GlassTrans);
                CreateSlider(GlassTrans, true);

                HSVColor ghsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                glassColor = new JSONStorableColor("Bottle Color", ghsvc, BottleColorCallback);
                RegisterColor(glassColor);
                CreateColorPicker(glassColor, true);

                HSVColor chsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                CapColor = new JSONStorableColor("Bottle Cap Color", chsvc, CapColorCallback);
                RegisterColor(CapColor);
                CreateColorPicker(CapColor, true);

                HSVColor bhsvc = HSVColorPicker.RGBToHSV(1f, 1f, 1f);
                BlandColor = new JSONStorableColor("Bottle Bland Color", bhsvc, BlandColorCallback);
                RegisterColor(BlandColor);
                CreateColorPicker(BlandColor, true);

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>About</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Custom Glass Bottle\n\n" +
                    "Liquid and bottle can be changed.\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

            }
            catch (Exception e)
            {
                SuperController.LogError("Exception caught: " + e);
            }
        }

        // Update is called with each rendered frame by Unity
        void Update()
        {
            // I don't know why the fuck this asset does not get the children properly during the loading process, so I'm checking the singleton
            // In reality, that's even cleaner to do it that way
            if (initLoops < maxInitLoops && editorInitialized == false && !SuperController.singleton.isLoading)
            {
                initLoops++;
                InitEditor();
            }

            if (initLoops == maxInitLoops)
            {
                SuperController.LogError("Liquid Glass Pack Bottle : This CUA could not be initialized properly. Please select a compatible object and reload the script.");
                initLoops++; // To avoid spamming the log with the error
            }

            // first set.
            if(first_loading == false && GlassRoot != null && LiquidRoot != null)
            {
                //SuperController.LogMessage(">> " + GlassTransComponents.val);
                //SuperController.LogMessage(">> ----------------------------");
                first_loading = true;
                ChangeGlassRenderModeCallback(GlassTransComponents);
            }

            // wobble
            if (LiquidRoot != null)
            {
                if (rend == null)
                {
                    rend = LiquidRoot.GetComponent<Renderer>();
                }
                if (rend == null) return;

                if (LiquidPhysicsComponents.val) LiquidPhysics.UpdateLiquidPhysics(LiquidRoot, rend);

                if (CurrScale != ObjectRoot.transform.parent.transform.localScale.y)
                {
                    //SuperController.LogMessage(ObjectRoot.transform.parent.name + ">>" + ObjectRoot.transform.parent.transform.localScale.y);
                    CurrScale = ObjectRoot.transform.parent.transform.localScale.y;
                    _liquid_opt.UpdateCurrScale(CurrScale);
                }

                float angle = Quaternion.Angle(Quaternion.Euler(new Vector3(0, 0, 0)), ObjectRoot.transform.parent.transform.rotation);
                if (CurrAngle != angle)
                {
                    CurrAngle = angle;
                    LiquidType.UpdateLiquidHeightAround(CurrAngle);
                    LiquidType.SetCurrType(CurrType);
                    _liquid_opt.UpdateHeight();
                }
            }

        }

        void OnDestroy()
        {
            RemoveToggle(GlassTransComponents);
        }

        private bool InitEditor()
        {
            if (editorInitialized == true) return true;

            ObjectRoot = getObjectRoot(containingAtom.reParentObject, "beer_bottle");
            if (ObjectRoot != null) CurrType = "beer_bottle";
            if (ObjectRoot == null) ObjectRoot = getObjectRoot(containingAtom.reParentObject, "soju_bottle");
            if (ObjectRoot != null && CurrType == null) CurrType = "soju_bottle";
            if (ObjectRoot == null) ObjectRoot = getObjectRoot(containingAtom.reParentObject, "wine_bottle");
            if (ObjectRoot != null && CurrType == null) CurrType = "wine_bottle";
            if (ObjectRoot == null) ObjectRoot = getObjectRoot(containingAtom.reParentObject, "plastic_bottle");
            if (ObjectRoot != null && CurrType == null) CurrType = "plastic_bottle";
            if (ObjectRoot == null) ObjectRoot = getObjectRoot(containingAtom.reParentObject, "blackjack_bottle");
            if (ObjectRoot != null && CurrType == null) CurrType = "blackjack_bottle";

            if (ObjectRoot == null)
            {
                CurrType = null;
                return false;
            }

            LiquidType.SetCurrType(CurrType);
            LiquidType.UpdateLiquidHeightAround(CurrAngle);

            LiquidRoot = getChildRoot(containingAtom.reParentObject, "liquid");
            GlassRoot = getChildRoot(containingAtom.reParentObject, "bottle");
            GlassCapRoot = getChildRoot(containingAtom.reParentObject, "cap");
            GlassSubCapRoot = getChildRoot(containingAtom.reParentObject, "cap_line");
            GlassbrandRoot = getChildRoot(containingAtom.reParentObject, "brand");
            //animator.speed = 0f;
            // I found my transforms, now the rest
            if (LiquidRoot)
            {
                editorInitialized = true;

                // Everything is initialized, triggering the callbacks to init the door rotation saved in the scene file
                // I'm just quickly fixing the script for VamTastic, I'm not gonna refactor everything, so in reality,
                // the float val param of the callbacks is useless

                _liquid_opt.SetLiquidRoot(LiquidRoot);
                _liquid_opt.UpdateValue();

                OnGlassAlphaChanged(GlassTrans.val);
                BottleColorCallback(glassColor);
                CapColorCallback(CapColor);
                BlandColorCallback(BlandColor);
                ChangeCapOnOffCallback(CapOnOffComponents);
                ChangeBrandOnOffCallback(BrandOnOffComponents);

                return true;
            }
            else
            {
                SuperController.LogError("Liquid Glass Pack Cup : your CUA is missing mandatory components.");
                initLoops = maxInitLoops; // We don't have what we need, we consider the initialization failed
                return false;
            }

            return false;
        }

        // Recursive function searching for a transform containing the string set in objectNameSearchString
        // It is used the first time for the initialization of the script
        // This script NEEDS to use a string "containing" your original because the cloning of the original asset will add "Clone" behind it.
        private Transform getObjectRoot(Transform parent, string rootName)
        {
            foreach (Transform child in parent)
            {
                if (child.name.Contains(rootName))
                {
                    return child;
                }
                else
                {
                    Transform childSearch = getObjectRoot(child, rootName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        private Transform getChildRoot(Transform parent, string searchName)
        {
            foreach (Transform child in parent)
            {
                if (child.name == searchName)
                {

                    return child;
                }
                else
                {
                    Transform childSearch = getChildRoot(child, searchName);
                    if (childSearch != null) return childSearch;
                }
            }

            return null;
        }

        // Glass Alpha
        void OnGlassAlphaChanged(float value)
        {
            if (GlassRoot == null) return;

            Renderer renderer = GlassRoot.GetComponent<Renderer>();
            Color cc = renderer.material.GetColor("_Color");
            cc.a = value;
            renderer.material.SetColor("_Color", cc);

            if (SubGlassRoot != null)
            {
                Renderer s_renderer = SubGlassRoot.GetComponent<Renderer>();
                Color s_cc = s_renderer.material.GetColor("_Color");
                //SuperController.LogMessage(">> s_cc " + s_cc);
                s_cc.a = value;
                s_renderer.material.SetColor("_Color", s_cc);
            }
            //SuperController.LogMessage(">> a " + cc.a);
        }

        //  material color
        protected void BottleColorCallback(JSONStorableColor dc)
        {
            if (GlassRoot == null) return;

            Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

            Renderer renderer = GlassRoot.GetComponent<Renderer>();
            c.a = renderer.material.color.a;
            renderer.material.SetColor("_Color", c);

            //SuperController.LogMessage(">> c " + c.a);
        }

        protected void ChangeGlassRenderModeCallback(JSONStorableBool jb)
        {
            if (jb.val)
                ChangeRenderMode(GlassRoot, "Transparent");
            else
                ChangeRenderMode(GlassRoot, "Opaque");
        }

        protected void ChangeLiquidPhysicsCallback(JSONStorableBool jb)
        {
            //if (jb.val)
            //    ChangeRenderMode(GlassRoot, "Transparent");
            //else
            //    ChangeRenderMode(GlassRoot, "Opaque");
        }

        protected void ChangeCapOnOffCallback(JSONStorableBool jb)
        {
            if (GlassCapRoot == null) return;
            if (jb.val)
                GlassCapRoot.gameObject.SetActive(true);
            else
                GlassCapRoot.gameObject.SetActive(false);
        }

        protected void ChangeBrandOnOffCallback(JSONStorableBool jb)
        {
            if (GlassbrandRoot == null) return;
            if (jb.val)
                GlassbrandRoot.gameObject.SetActive(true);
            else
                GlassbrandRoot.gameObject.SetActive(false);
        }

        //  Cap color
        protected void CapColorCallback(JSONStorableColor dc)
        {
            if (GlassCapRoot == null) return;

            Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

            Renderer renderer = GlassCapRoot.GetComponent<Renderer>();
            //c.a = renderer.material.color.a;
            renderer.material.SetColor("_Color", c);

            if (GlassSubCapRoot != null)
            {
                Renderer s_renderer = GlassSubCapRoot.GetComponent<Renderer>();
                //c.a = renderer.material.color.a;
                s_renderer.material.SetColor("_Color", c);
            }

            //SuperController.LogMessage(">> c " + c.a);
        }

        //  Bland color
        protected void BlandColorCallback(JSONStorableColor dc)
        {
            if (GlassbrandRoot == null) return;

            Color c = Color.HSVToRGB(dc.valNoCallback.H, dc.valNoCallback.S, dc.valNoCallback.V);

            Renderer renderer = GlassbrandRoot.GetComponent<Renderer>();
            //c.a = renderer.material.color.a;
            renderer.material.SetColor("_Color", c);

            //SuperController.LogMessage(">> c " + c.a);
        }

        protected void ChangeRenderMode(Transform target, string blendMode)
        {
            if (target == null)
            {
                return;
            }
            Renderer renderer = target.GetComponent<Renderer>();

            if (blendMode == "Opaque")
            {
                renderer.material.SetFloat("_Mode", 0.0f);
                renderer.material.SetOverrideTag("RenderType", "Opaque");
                renderer.material.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.One);
                renderer.material.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.Zero);
                renderer.material.SetInt("_ZWrite", 1);
                renderer.material.DisableKeyword("_ALPHATEST_ON");
                renderer.material.DisableKeyword("_ALPHABLEND_ON");
                renderer.material.DisableKeyword("_ALPHAPREMULTIPLY_ON");
                renderer.material.renderQueue = -1;
            }
            else
            {
                renderer.material.SetFloat("_Mode", 3.0f);
                renderer.material.SetOverrideTag("RenderType", "Transparent");
                renderer.material.SetInt("_SrcBlend", (int)UnityEngine.Rendering.BlendMode.One);
                renderer.material.SetInt("_DstBlend", (int)UnityEngine.Rendering.BlendMode.OneMinusSrcAlpha);
                renderer.material.SetInt("_ZWrite", 0);
                renderer.material.DisableKeyword("_ALPHATEST_ON");
                renderer.material.DisableKeyword("_ALPHABLEND_ON");
                renderer.material.EnableKeyword("_ALPHAPREMULTIPLY_ON");
                renderer.material.renderQueue = 3000;
            }
        }

    }

}