using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

public static class LiquidPhysics
{
    private static float MaxWobble = 0.03f;
    private static float WobbleSpeed = 5.0f;
    private static float RecoveryRate = 1f;

    private static Vector3 prevPos;
    private static Vector3 prevRot;
    private static float wobbleAmountToAddX;
    private static float wobbleAmountToAddZ;

    public static void UpdateLiquidPhysics(Transform LiquidRoot, Renderer rend)
    {
        wobbleAmountToAddX = Mathf.Lerp(wobbleAmountToAddX, 0, Time.deltaTime * RecoveryRate);
        wobbleAmountToAddZ = Mathf.Lerp(wobbleAmountToAddZ, 0, Time.deltaTime * RecoveryRate);

        // make a sine wave of the decreasing wobble
        float wobbleAmountX = wobbleAmountToAddX * Mathf.Sin(WobbleSpeed * Time.time);
        float wobbleAmountZ = wobbleAmountToAddZ * Mathf.Sin(WobbleSpeed * Time.time);

        // send it to the shader
        rend.material.SetFloat("_WobbleX", wobbleAmountX);
        rend.material.SetFloat("_WobbleZ", wobbleAmountZ);

        // Move Speed
        Vector3 moveSpeed = (prevPos - LiquidRoot.position) / Time.deltaTime;
        Vector3 rotationDelta = LiquidRoot.rotation.eulerAngles - prevRot;

        // add clamped speed to wobble
        wobbleAmountToAddX += Mathf.Clamp((moveSpeed.x + (rotationDelta.z * 0.2f)) * MaxWobble, -MaxWobble, MaxWobble);
        wobbleAmountToAddZ += Mathf.Clamp((moveSpeed.z + (rotationDelta.x * 0.2f)) * MaxWobble, -MaxWobble, MaxWobble);

        // save the last position
        prevPos = LiquidRoot.position;
        prevRot = LiquidRoot.rotation.eulerAngles;

    }


}